/* -----------------------------------------------------------------------

   Copyright (C) 2004 Mariusz "Manio" Bialonczyk

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   ----------------------------------------------------------------------- */

#include "LCD.h"

CallerInfo LCDcallerinfo;

LCD::LCD()
{
	iSock=-1;
	iCallers=0;
	bVisible=false;
	bzero(LCDcallerinfo.szTime,TIME_STR_SIZE);
	bzero(szBuff,LCD_BUFF_SIZE);
	bzero(szRings,RINGS_STR_SIZE);
}

LCD::~LCD()
{
	if (iSock>0)
		SockClose();
}

void LCD::FormatTime(tm* rtime)
{
	snprintf(LCDcallerinfo.szTime, TIME_STR_SIZE, "%02d:%02d:%02d", rtime->tm_hour, rtime->tm_min, rtime->tm_sec);
}

void LCD::Prepare()
{
	lprintf("LCD::Prepare\n");
	bVisible=1;
	if (iCallers==0)	// making screen first time
	{
		snprintf(szBuff,LCD_BUFF_SIZE,"screen_add caller\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"screen_set caller -priority 16\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller title string\n");
		SockSendBuff();
		//niepene kratki:
// 		snlprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller title 1 1 {%c%c%c clipd status %c%c}\n",127,127,127,127,127);
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller title 1 1 {%c%c%c clipd status %c%c}\n",14,14,14,14,14);
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller mark_total string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller caller_1 string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller caller_2 string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller time string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller rings string\n");
		SockSendBuff();
	}
	if (iCallers>0)		// clear previous data
	{
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller mark_total\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller mark_total string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller caller_1\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller caller_1 string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller caller_2\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller caller_2 string\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller rings\n");
		SockSendBuff();
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller rings string\n");
		SockSendBuff();
	}
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller caller_2 1 3 {waiting for data...}\n");
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller time 1 4 {%s}\n",LCDcallerinfo.szTime);
	SockSendBuff();
}

void LCD::UpdateRings(int iRings)
{
	lprintf("LCD::UpdateRings\n");
	int iPos=12;
	snprintf(szRings,RINGS_STR_SIZE,"%d ring%s",iRings,(iRings!=1?"s":""));
	if (iRings<10)
		iPos++;
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller rings %d 4 {%s}\n",iPos,szRings);
	SockSendBuff();
}

unsigned short int LCD::NoData()
{
	iCallers++;
	lprintf("LCD::NoData: ++iCallers=%d\n",iCallers);
	if (!bVisible)
		Prepare();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller caller_1\n");
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_add caller caller_1 string\n");
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller caller_2 1 3 {no CID data catched}\n",LCDcallerinfo.szLCDLine2);
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller mark_total %d 2 {%d}\n",iCallers>9?19:20,iCallers);
	SockSendBuff();
	return iCallers;
}

unsigned short int LCD::Add()
{
	iCallers++;
	lprintf("LCD::Add: ++iCallers=%d\n",iCallers);
	if (!bVisible)
		Prepare();
	if (*LCDcallerinfo.szLCDLine1!=0)
	{
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller caller_1 1 2 {%s}\n",LCDcallerinfo.szLCDLine1);
	}
	else
		snprintf(szBuff,LCD_BUFF_SIZE,"widget_del caller caller_1\n",LCDcallerinfo.szLCDLine1);
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller caller_2 1 3 {%s}\n",LCDcallerinfo.szLCDLine2);
	SockSendBuff();
	snprintf(szBuff,LCD_BUFF_SIZE,"widget_set caller mark_total %d 2 {%d}\n",iCallers>9?19:20,iCallers);
	SockSendBuff();
	UpdateRings(gRings);
	return iCallers;
}

void LCD::Clear()
{
	lprintf("LCD::Clear\n");
	if (bVisible)
	{
		bVisible=false;
		snprintf(szBuff,LCD_BUFF_SIZE,"screen_del caller\n");
		SockSendBuff();
	}
}

bool LCD::Connect()
{
	lprintf("LCD::Connect\n");
	usleep(500000);					// wait for the server to start up
	SockConnect();
	if (iSock<=0)
	{
		lprintf("LCD::Connect: error connecting to %s on port %d\n",SERVERNAME,LCDPORT);
		return false;
	}
	strcpy(szBuff,"hello\n");
	SockSendBuff();
	usleep(500000);					// wait for the server to say hi.
	strcpy(szBuff,"client_set -name clipd\n");
	SockSendBuff();
	return true;
}

unsigned short int LCD::ResetCallers()
{
	lprintf("LCD::ResetCallers\n");
	return iCallers=0;
}

int LCD::SockSendBuff()
{
	int iLen;
	int iOffset=0;

	iLen=strlen(szBuff);
	while (iOffset!=iLen)
	{
		// write isn't guaranteed to send the entire string at once,
		// so we have to sent it in a loop like this
		int iSent=write(iSock, szBuff+iOffset, iLen-iOffset);
		if (iSent==-1)
		{
			if (errno != EAGAIN)
			{
				lprintf("LCD::SockSendBuff: socket write error\n");
				//shutdown(fd, 2);
				return iSent;
			}
			continue;
		}
		else if (iSent==0)
		{
			// when this returns zero, it generally means
			// we got disconnected
			return iSent+iOffset;
		}
		iOffset+=iSent;
	}
	szBuff[strlen(szBuff)-1]=0;
	lprintf("LCD::SockSendBuff: '%s'\n",szBuff);
}

bool LCD::SockConnect()
{
	struct sockaddr_in servername;
	struct hostent *hostinfo;
	int iRetVal=0;

	lprintf("LCD::SockConnect: creating socket to LCDd ... ");
	iSock=socket(PF_INET, SOCK_STREAM, 0);
	gDate=false;
	if (iSock<0)
	{
		lprintf("failed\n");
		return false;
	}
	lprintf("success\n");
	gDate=true;

	//---- initing sockaddr
	memset(&servername,0,sizeof(servername));
	servername.sin_family = AF_INET;
	servername.sin_port = htons(LCDPORT);
	hostinfo = gethostbyname(SERVERNAME);
	if (hostinfo == NULL)
	{
		lprintf("LCD::SockConnect: unknown host\n");
		return false;
	}
	servername.sin_addr = *(struct in_addr*)hostinfo->h_addr;
	//----

	iRetVal=connect(iSock, (struct sockaddr*)&servername, sizeof(servername));
	if (iRetVal<0)
	{
		lprintf("LCD::SockConnect: connect failed\n");
		shutdown(iSock,2);
		return false;		// Normal exit if server doesn't exist...
	}
	fcntl(iSock, F_SETFL, O_NONBLOCK);
	return true;
}

int LCD::SockClose()
{
	lprintf("LCD::SockClose\n");
	int iRetVal;
	iRetVal=shutdown(iSock,2);
	if (!iRetVal)
	{
		close(iSock);
		iSock=-1;
	}
	return iRetVal;
}
