/* -----------------------------------------------------------------------

   Copyright (C) 2004 Mariusz "Manio" Bialonczyk

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   ----------------------------------------------------------------------- */

#include "Modem.h"

Modem::Modem()
{
	fd=0;
}

void Modem::Open()
{
	lprintf("Modem::Open: opening '%s'... ",MODEM_DEVICE);
	gDate=false;
	if ((fd = open(MODEM_DEVICE, O_RDWR | O_NOCTTY | O_SYNC)) < 0)
	{
		lprintf("error\n");
		gDate=true;
	}
	else
	{
		lprintf("success\n");
		gDate=true;
	}
}

void Modem::Init()
{
	lprintf("Modem::Init: initializing modem\n");
	tcflush(fd, TCIOFLUSH);	//flushing all
	write(fd,&szATZ,strlen(szATZ));
	sleep(1);
	if (bFormat==0)	//formatted
		write(fd,&szInitString1,strlen(szInitString1));
	else		//RAW
		write(fd,&szInitString2,strlen(szInitString2));
	sleep(1);
}

void Modem::DeInit()
{
	lprintf("Modem::DeInit: resetting modem\n");
	write(fd,&szATZ,strlen(szATZ));
	sleep(1);
	write(fd,&szDeInitString,strlen(szDeInitString));
	sleep(1);
}

Modem::~Modem()
{
	DeInit();
	if (fd)
		Close();
}

void Modem::Close()
{
	lprintf("Modem::Close: closing '%s'\n",MODEM_DEVICE);
	if (fd)
	{
		close(fd);
		fd=0;
	}
}

bool Modem::BlockingRead()
{
	int iRetVal;
	bzero(&szModemBuffer,MODEM_BUFF_SIZE);
	bCanDial=true;
	iRetVal=read(fd,szModemBuffer,MODEM_BUFF_SIZE);
	bCanDial=false;
	if (iRetVal==-1)
		return false;
	else
		return true;
}

bool Modem::Opened()
{
	if (fd==0)
		return false;
	else
		return true;
}

void Modem::Dial(char *szNumber)
{
	//check for and delete '\n' and '\r'
	for (int i=0;i<strlen(szNumber);i++)
	{
		if (szNumber[i]=='\n' || szNumber[i]=='\r')
		{
			szNumber[i]==0;
			break;
		}
	}
	if (szNumber[0]=='0' && szNumber[1]=='3' && szNumber[2]=='2')	//if prefix is 032
		szNumber+=3;	//just skip it
	lprintf("Modem::Dial: dialing number: '%s'\n",szNumber);
	sprintf(szModemBuffer,"ATS11=50DT%s;h\r\n",szNumber);
	write(fd,&szModemBuffer,strlen(szModemBuffer));
	bzero(&szModemBuffer,MODEM_BUFF_SIZE);	//clean buffer because main program is in Modem::BlockingRead()
}

int Modem::Parse()
{
	if (fd==0 || !BlockingRead())
		return PARSE_READ_ERROR;
	else
	{
#ifdef DEBUG_INFO
		int i=0;
		lprintf("Modem::Parse: get: [");
		gDate=false;
		while (szModemBuffer[i]!=0 && szModemBuffer[i]!=0xa)
			lprintf("%c",szModemBuffer[i++]);
		lprintf("] = ");
		i=0;
		while (szModemBuffer[i]!=0)
			lprintf("0x%.2x ",szModemBuffer[i++]);
		lprintf("\n");
		gDate=true;
#endif
		if (pCIDNumberStart=strstr(szModemBuffer,bFormat==0 ? "NMBR" : "801601"))
		{
			bUnlisted=false;
			return PARSED_CALLERID;
		}
		else if (bFormat==1 && (pCIDNumberStart=strstr(szModemBuffer,"800D01")))
		{
			bUnlisted=true;
			return PARSED_CALLERID;
		}
		else if (strstr(szModemBuffer,"RING"))
			return PARSED_RING;
		return PARSED_NOTHING;
	}
}

void Modem::GetCIDNumber(char* szNumber)
{
	lprintf("Modem::GetCIDNumber\n");
	int iDst=0;
	bzero(szNumber,sizeof(szNumber));				//clear buffer
	if (bFormat==0)	//formatted
	{
		pCIDNumberStart+=7;					//beginning of the number
		while (*pCIDNumberStart!='\n' && *pCIDNumberStart!=0)
			szNumber[iDst++]=*pCIDNumberStart++;
	}
	else		//RAW
	{
		if (bUnlisted)
			strcpy(szNumber,"0000000000");
		else
		{
			pCIDNumberStart=strstr(pCIDNumberStart,"020A");		//start of number is two bytes forward and it's finished with "BF"
			pCIDNumberStart+=5;					//beginning of the number
			while (*(pCIDNumberStart+1)!='\n' && *(pCIDNumberStart+1)!=0)
			{
				szNumber[iDst++]=*pCIDNumberStart;
				pCIDNumberStart+=2;
			}
		}
	}
	if (!bUnlisted)
		lprintf("Modem::GetCIDNumber: parsed number is: %s\n",szNumber);
	else
		lprintf("Modem::GetCIDNumber: unlisted number\n",szNumber);
}
