/* -----------------------------------------------------------------------

   Copyright (C) 2004 Mariusz "Manio" Bialonczyk

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   ----------------------------------------------------------------------- */

#include "MySQL.h"

MySQL::MySQL()
{
	last_id=0;
}

MySQL::~MySQL()
{
	Disconnect();
	mysql_server_end();
}

bool MySQL::Connect()
{
	mysql_init(&mysql);
	if (!&mysql)
	{
		lprintf("MySQL::Connect failed: no memory\n");
		return false;
	}
	mysql_options(&mysql,MYSQL_OPT_COMPRESS,0);
	if (!mysql_real_connect(&mysql, MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWD, "clip", 0, NULL, 0))
	{
    		lprintf( "MySQL::Connect: Failed to connect to database: Error: %s\n",mysql_error(&mysql));
		return false;
	}
	return true;
}

void MySQL::Disconnect()
{
	mysql_close(&mysql);
}

bool MySQL::Add(Connection* CurCon)
{
#ifdef DEBUG_INFO
	lprintf("MySQL::Add\n");
#endif
	if (!CheckConnection())
		return false;
	tm *rtime=CurCon->GetTime();
	snprintf(szQuery,QUERY_SIZE,"INSERT INTO connection_history VALUES ('%s','%04d-%02d-%02d %02d:%02d:%02d',%d,%d,NULL)",
		CurCon->GetNumberBuffAddr(),
		rtime->tm_year+1900, rtime->tm_mon+1, rtime->tm_mday, rtime->tm_hour, rtime->tm_min, rtime->tm_sec,
		rtime->tm_wday,
		CurCon->GetRings()
	);
	if (mysql_query(&mysql,szQuery) != 0)
		goto err;
	if (mysql_error(&mysql)[0] == 0 && mysql_insert_id(&mysql) != 0)
		last_id = mysql_insert_id(&mysql);
	return true;
err:
	lprintf("MySQL::Add failed: %s\n", mysql_error(&mysql));
	return false;
}

bool MySQL::CheckConnection()
{
#ifdef DEBUG_INFO
	lprintf("MySQL::CheckConnection\n");
#endif
	if (!mysql_ping(&mysql))
	{
#ifdef DEBUG_INFO
		lprintf("MySQL::CheckConnection: mysqld is alive\n");
#endif
	}
	else
	{
		if (mysql_errno(&mysql) == CR_SERVER_GONE_ERROR)
		{
			if (!mysql_ping(&mysql))
				lprintf("MySQL::CheckConnection: connection was down, but mysqld is now alive");
		}
		else
		{
			lprintf("MySQL::CheckConnection: mysqld doesn't answer to ping, error: '%s'\n",mysql_error(&mysql));
			return false;
		}
	}
	return true;
}

bool MySQL::UpdateNewCallers()
{
	if (!CheckConnection())
		return false;
	if (snprintf(szQuery,QUERY_SIZE,"UPDATE new_callers SET value=value+1 WHERE ID=1")==-1)
	{
		lprintf("MySQL::UpdateNewCallers failed: query buffer size truncated\n");
		return false;
	}
	if (mysql_query(&mysql,szQuery) != 0)
		goto err;
	lprintf("MySQL::UpdateNewCallers returned true\n");
	return true;
err:
	lprintf("MySQL::UpdateNewCallers failed: %s\n", mysql_error(&mysql));
	return false;
}

bool MySQL::UpdateRings(int iRings)
{
	if (last_id==0)
	{
		lprintf("MySQL::UpdateRings failed: no last_id so don't know whose rings needs to be updated\n");
		return false;
	}
	if (!CheckConnection())
		return false;
	if (snprintf(szQuery,QUERY_SIZE,"UPDATE connection_history SET rings=%d WHERE ID=%d",iRings,last_id)==-1)
	{
		lprintf("MySQL::UpdateRings failed: query buffer size truncated\n");
		return false;
	}
	if (mysql_query(&mysql,szQuery) != 0)
		goto err;
	return true;
err:
	lprintf("MySQL::UpdateRings failed: %s\n", mysql_error(&mysql));
	return false;
}

bool MySQL::SearchAndFill(CallerInfo* Dest, char* szNumber)
{
#ifdef DEBUG_INFO
	lprintf("MySQL::SearchAndFill\n");
#endif
	if (!CheckConnection())
		return false;
	if (snprintf(szQuery,QUERY_SIZE,"select * from callers where number=%s",szNumber)==-1)
	{
		lprintf("MySQL::SearchAndFill failed: query buffer size truncated\n");
		return false;
	}
	if (mysql_query(&mysql,szQuery) != 0)
		goto err;
	mysql_field_count(&mysql);

	MYSQL_RES	*res;
	MYSQL_ROW	row, end_row;
	int num_fields;

	if (!(res = mysql_store_result(&mysql)))
		goto err;
	num_fields = mysql_num_fields(res);
	if (row = mysql_fetch_row(res))
	{
		bzero(Dest->szLCDLine1,LCD_LINE_WIDTH+1);
		bzero(Dest->szLCDLine2,LCD_LINE_WIDTH+1);
		row++;	//lcdline1
		if (row)
			strncpy(Dest->szLCDLine1, (char*)*row, LCD_LINE_WIDTH);
		row++;	//lcdline2
		if (row)
			strncpy(Dest->szLCDLine2, (char*)*row, LCD_LINE_WIDTH);
	}
	else	//not found
	{
		mysql_free_result(res);
		//copy phone number only:
	//	*Dest->szLCDLine1=0;
	//	strcpy(Dest->szLCDLine1,"<<unknown caller>>");
		if (strstr(szNumber,"033801"))				//compare phone booth prefix
			strcpy(Dest->szLCDLine1,"booth: (area 33)");	//unknown phone booth from area 33
		else if (strstr(szNumber,"032805"))
			strcpy(Dest->szLCDLine1,"booth: (area 32)");	//unknown phone booth from area 32
		else
			strcpy(Dest->szLCDLine1,"not found:");
		strncpy(Dest->szLCDLine2,szNumber,LCD_LINE_WIDTH);
		return false;
	}
	mysql_free_result(res);
	return true;
err:
	lprintf("MySQL::SearchAndFill failed: %s\n", mysql_error(&mysql));
	return false;
}
