/* -----------------------------------------------------------------------

   Copyright (C) 2004 Mariusz "Manio" Bialonczyk

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   ----------------------------------------------------------------------- */

#include "main.h"
#include "LCD.h"
#include "Connection.h"
#include "Modem.h"
#include "MySQL.h"

int gRings;
unsigned short int gCallers;
pthread_t thread;
bool bIdleRingState=true;
bool bGotCIDData=false;
bool bCanDial=false;
int fifo;		//'/dev/clipd' FIFO descriptor
bool gDate=true;	//if true the lprintf add a datestamp in the beginning of the line
char* pFIFOBuffer;

//global classes:
	Modem *modem=new Modem;
	Connection *connection=new Connection;
	MySQL *SQL=new MySQL;
	LCD *cLCD=new LCD;

void lprintf(char *fmt, ...)
{
	static FILE *log;
	va_list ap;

	static char *shortdays[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	static char *shortmonths[] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

	static time_t thetime;
	struct tm *rtime;
	if (gDate)
	{
		time (&thetime);
		rtime = localtime (&thetime);
	}

 	log=fopen(LOG_FILE,"a");
	va_start(ap, fmt);
	if (gDate)
		fprintf(log, "%s %s %02d %d %02d:%02d:%02d ", shortdays[rtime->tm_wday], shortmonths[rtime->tm_mon], rtime->tm_mday, rtime->tm_year+1900, rtime->tm_hour, rtime->tm_min, rtime->tm_sec);
	vfprintf(log, fmt, ap);
	va_end(ap);
	fclose(log);
}

extern "C" void* w8_4nextring()
{
	int iPrevRings=gRings;
	int iPrevCaller=gCallers;
	bIdleRingState=false;
	sleep(RINGS_DELAY);
	if ( (iPrevRings==gRings && iPrevCaller==gCallers) || (iPrevRings==gRings && iPrevCaller==(gCallers-1)) )
	// gCallers-1, because without another ring could get CIDdata only or no data was catched (gCallers also incremented)
	{
		bIdleRingState=true;
		bGotCIDData=false;
		lprintf("setting bIdleRingState to true\n");
	}
	return(NULL);
}

char* FIFORead()
{
	static char szFIFOBuffer[31];
	int iRetVal;
	if ((fifo = open(FIFO_DEVICE, O_RDONLY)) < 0)
	{
		lprintf("FIFORead: opening fifo '%s' failed: %s\n",FIFO_DEVICE,strerror(errno));
		fifo=-1;
		return 0;
	}
	bzero(szFIFOBuffer,sizeof(szFIFOBuffer));
	iRetVal=read(fifo,&szFIFOBuffer,sizeof(szFIFOBuffer));
	close(fifo);
	fifo=-1;
	if (iRetVal==-1)
	{
		lprintf("FIFORead: read error: %s\n",strerror(errno));
		return 0;
	}
	if (iRetVal==0)		//EOF
		return 0;
	return szFIFOBuffer;
}

void signal_handler(int sig)
{
	switch(sig)
	{
		case SIGUSR1:				//clear_lcd
			cLCD->Clear();
			gCallers=cLCD->ResetCallers();
			break;
		case SIGUSR2:				//read a fifo and dial a number
			pFIFOBuffer=FIFORead();
			if (pFIFOBuffer && bCanDial)
				modem->Dial(pFIFOBuffer);
			break;
		case SIGABRT:
			lprintf("ABRT signal catched - closing modem\n",1);
			modem->DeInit();
			modem->Close();
			break;
		case SIGHUP:
			lprintf("HUP signal catched - initializing modem\n",1);
			if (modem->Opened())
				modem->Close();
			modem->Open();
			modem->Init();
			break;
		case SIGTERM:
			lprintf("TERM signal catched - exiting\n");
			if (fifo)
				close(fifo);
			delete SQL;
			delete cLCD;
			delete connection;
			delete modem;
			modem=0;	//set pointer to zero for check in main loop if was a TERM signal
			break;
	}
}

void daemonize()
{
	int lfp,i;
	char str[10];
	
	if (getppid()==1)	// already a daemon
		return;
	i=fork();
	if (i<0)	// fork error
		exit(1);
	if (i>0)	// **** parent ****
		exit(0);

	// i==0, **** child (daemon) continues
	setsid();					// obtain a new process group
	for (i=getdtablesize();i>=0;--i)		// close all descriptors
		close(i);
	i=open("/dev/null",O_RDWR); dup(i); dup(i);	// handle standart I/O
	umask(027);					// set newly created file permissions
	lfp=open(LOCK_FILE,O_RDWR|O_CREAT,0640);
	if (lfp<0)					// can not open
		exit(1);
	if (lockf(lfp,F_TLOCK,0)<0)			// can not lock
		exit(0);

	// first instance continues
	sprintf(str,"%d\n",getpid());
	write(lfp,str,strlen(str));			// record pid to lockfile
	signal(SIGCHLD,SIG_IGN);			// ignore child
	signal(SIGTSTP,SIG_IGN);			// ignore tty signals
	signal(SIGTTOU,SIG_IGN);
	signal(SIGTTIN,SIG_IGN);
	signal(SIGUSR1,signal_handler);			// catch USR1 signal
	signal(SIGUSR2,signal_handler);			// catch USR2 signal
	signal(SIGHUP,signal_handler);			// catch HUP signal
	signal(SIGABRT,signal_handler);			// catch ABORT signal
	signal(SIGTERM,signal_handler);			// catch TERM signal
}

int main(int argc, char **argv)
{
	daemonize();
	lprintf("starting clipd %s\n",VERSION);

	modem->Open();
	modem->Init();
	SQL->Connect();
	cLCD->Connect();
	int iParseResult;
	while(1)
	{
		iParseResult=modem->Parse();
		switch (iParseResult)
		{
			case PARSE_READ_ERROR:
				if (modem==0)	//was a TERM signal
					break;
				if (modem->Opened())
					lprintf("Reading from modem error\n");
				sleep(5);
				continue;
			case PARSED_CALLERID:
				bGotCIDData=true;
				bIdleRingState=false;
				modem->GetCIDNumber(connection->GetNumberBuffAddr());
				SQL->SearchAndFill(&LCDcallerinfo,connection->GetNumberBuffAddr());	//searching caller data in mysql
				gCallers=cLCD->Add();							//add to LCD
				SQL->Add(connection);							//add to connection_history
				if (connection->NotifyOlo())
					connection->UDP_Send();
				SQL->UpdateNewCallers();
				continue;
			case PARSED_RING:
				gRings=connection->IncRings();
				if (bIdleRingState)
				{
					connection->ResetCounters();
					gRings=1;
					cLCD->FormatTime(connection->GetTime());
					cLCD->Prepare();	//on LCD: waiting for data
					bGotCIDData=false;
				}
				else
					SQL->UpdateRings(gRings);
				if (gRings==2 && !bIdleRingState && !bGotCIDData)	//no CID data
					gCallers=cLCD->NoData();
				pthread_create(&thread, NULL, w8_4nextring, NULL);
				cLCD->UpdateRings(gRings);
				continue;
			case PARSED_NOTHING:	//no essential data
				continue;
			default:
				lprintf("Error: unknown parse return value!!!\n");
				sleep(3);
				continue;
		}
	}
}
